#include <windows.h>
#include <windowsx.h>
#include <stdlib.h>
#include "resource.h"

//   
const DWORD ID_NEWGAME	= 102;
const DWORD ID_EXIT		= 103;
const DWORD ID_RULES	= 104;

//    
typedef enum { GBCS_NORMAL = 0, GBCS_SELECTED, GBCS_DISABLED } _GBCS;

//   
typedef struct {
	int number;					//    (0 -   )
	_GBCS state;				//  
} _GameBoard;

const int BOARD_WIDTH	= 10;	//      -
const int BOARD_HEIGHT	= 10;	//      -
const int CELL_SIZE		= 30;	//      ()
const int BACK_SIZE		= 3;	// - 
int CursorX, CursorY;			//     
int SelectX, SelectY;			//    
int Summ;						//    
HBRUSH hbrNormal;				//     
HBRUSH hbrSelected;				//     
HBRUSH hbrBackground;			//     
HBRUSH hbrCursor;				//  
HFONT hfntCell;					//  
SYSTEMTIME	stStart;			//   
bool GameComplete;				//   
int VictoryCount;				//  
HDC hMemDC;						//     
HBITMAP hMemBitmap;				//     
HBITMAP hMemBitmapOld;			//       
HBITMAP hbmpImg [BACK_SIZE];	// 
const LPCTSTR szTitle = " \"\"";		//  
_GameBoard GameBoard [BOARD_HEIGHT][BOARD_WIDTH];	//  

int GetGameTime ();
bool TestCellH (int x, int y);
bool TestCellV (int x, int y);
bool FillGameBoardCell (int x, int y);
void FillGameBoard ();
void DrawCell (int x, int y);
void DrawGameBoard ();
void DrawBackground (HDC hDC, const RECT* rc);
bool ClickCell (int x, int y);
LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE, LPTSTR, int);

//   
int GetGameTime ()
{
	//   
	SYSTEMTIME stCurrent;
	GetLocalTime (&stCurrent);
	//        
    int elapsed = 
		(stCurrent.wSecond + (stCurrent.wMinute + stCurrent.wHour * 60) * 60) -
		(stStart.wSecond + (stStart.wMinute + stStart.wHour * 60) * 60);
	if (elapsed < 0)
		//  -   
		elapsed = 24 * 60 * 60 + elapsed;
	return elapsed;
}

//     
void DrawCell (int x, int y)
{
	//       
	RECT rc = { x * CELL_SIZE, y * CELL_SIZE, (x + 1) * CELL_SIZE, (y + 1) * CELL_SIZE };
	DrawBackground (hMemDC, &rc);
	switch (GameBoard [y][x].state) {
		case GBCS_NORMAL:
		{
			HBRUSH hbrOld = SelectBrush (hMemDC, hbrNormal);
			Rectangle (hMemDC, rc.left, rc.top, rc.right - 1, rc.bottom - 1);
			SelectBrush (hMemDC, hbrOld);
			break;
		}
		case GBCS_SELECTED:
		{
			HBRUSH hbrOld = SelectBrush (hMemDC, hbrSelected);
			Rectangle (hMemDC, rc.left, rc.top, rc.right - 1, rc.bottom - 1);
			SelectBrush (hMemDC, hbrOld);
			break;
		}
	}
	//  
	if (GameBoard [y][x].state != GBCS_DISABLED) {
		char txt [33];
		wsprintf (txt, "%d", GameBoard [y][x].number);
		SetTextColor (hMemDC, RGB (0, 0, 0));
		int old = SetBkMode (hMemDC, TRANSPARENT);
		HFONT hfntOld = SelectFont (hMemDC, hfntCell);
		DrawText (hMemDC, txt, -1, &rc, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
		SelectFont (hMemDC, hfntOld);
		SetBkMode (hMemDC, old);
	}
}

//  
void DrawBackground (HDC hDC, const RECT* rc)
{
	HDC hTmpDC = CreateCompatibleDC (hDC);
	HBITMAP hTmpBitmapOld = SelectBitmap (hTmpDC, hbmpImg [VictoryCount % BACK_SIZE]);
	FillRect (hDC, rc, hbrBackground);
	BitBlt (hDC, rc->left, rc->top, rc->right - rc->left, rc->bottom - rc->top,
		hTmpDC, rc->left, rc->top, SRCCOPY); 
	SelectBitmap (hTmpDC, hTmpBitmapOld);
	DeleteDC (hTmpDC);
}

//       
void DrawGameBoard ()
{
	//   
	const RECT rc = { 0, 0, BOARD_HEIGHT * CELL_SIZE - 1, BOARD_WIDTH * CELL_SIZE - 1 };
	DrawBackground (hMemDC, &rc);
	//   
	for (int y = 0; y < BOARD_HEIGHT; ++y)
		for (int x = 0; x < BOARD_WIDTH; ++x)
			DrawCell (x, y);
}

//      
bool ClickCell (int x, int y)
{
	if (SelectX == -1) {
		if (GameBoard [y][x].state == GBCS_NORMAL) {
			//   
			GameBoard [y][x].state = GBCS_SELECTED;
			SelectX = x;
			SelectY = y;
			DrawCell (x, y);
		}
	} else {
		if (SelectX == x && SelectY == y) {
			//    
			GameBoard [y][x].state = GBCS_NORMAL;
			SelectX = -1;
			SelectY = -1;
			DrawCell (x, y);
		} else 
			if (GameBoard [y][x].state == GBCS_NORMAL &&
				((SelectX == x && (SelectY == y + 1 || SelectY == y - 1)) ||
				((SelectX == x + 1 || SelectX == x - 1) && SelectY == y)) &&
				(GameBoard [y][x].number + GameBoard [SelectY][SelectX].number == 10)) {
				//   
				GameBoard [y][x].state = GBCS_DISABLED;
				GameBoard [SelectY][SelectX].state = GBCS_DISABLED;
				DrawCell (x, y);
				DrawCell (SelectX, SelectY);
				SelectX = -1;
				SelectY = -1;
				Summ += 2;
			}
	}
	return (Summ == BOARD_WIDTH * BOARD_HEIGHT);
}

//    
bool TestCellH (int x, int y)
{
	GameBoard [y][x].number = (rand () * 5) / RAND_MAX + 1;
	GameBoard [y][x + 1].number = 10 - GameBoard [y][x].number;
	//    
	if (x + 1 == BOARD_WIDTH) {
		if (y + 1 == BOARD_HEIGHT)
			return true;
		else
			if (FillGameBoardCell (0, y + 1))
				return true;
	} else
		if (FillGameBoardCell (x + 1, y))
			return true;
	//  
	GameBoard [y][x].number = 0;
	GameBoard [y][x + 1].number = 0;
	return false;
}

//    
bool TestCellV (int x, int y)
{
	GameBoard [y][x].number = (rand () * 5) / RAND_MAX + 1;		// 1-5
	GameBoard [y + 1][x].number = 10 - GameBoard [y][x].number;
	//    
	if (x + 1 == BOARD_WIDTH) {
		if (y + 1 == BOARD_HEIGHT)
			return true;
		else
			if (FillGameBoardCell (0, y + 1))
				return true;
	} else
		if (FillGameBoardCell (x + 1, y))
			return true;
	//  
	GameBoard [y][x].number = 0;
	GameBoard [y + 1][x].number = 0;
	return false;
}

//      
bool FillGameBoardCell (int x, int y)
{
	//      (x,y) 
	if (GameBoard [y][x].number == 0) {
		//       
		bool h = (x + 1 != BOARD_WIDTH  && GameBoard [y][x + 1].number == 0);
		bool v = (y + 1 != BOARD_HEIGHT && GameBoard [y + 1][x].number == 0);
		//    , ,
		//     ,   
		if ((h && !v) || (h && v && ((rand () * 2) / RAND_MAX) == 1)) {
			//    
			if (TestCellH (x, y))
				return true;
			h = false;
		}
		//   
		if (v) {
			//    
			if (TestCellV (x, y))
				return true;
			if (h)
				//    
				if (TestCellH (x, y))
					return true;
		}
		//   
		return false;
	}
	//    
	if (x + 1 == BOARD_WIDTH) {
		if (y + 1 == BOARD_HEIGHT)
			return true;
		else
			return FillGameBoardCell (0, y + 1);
	} else
		return FillGameBoardCell (x + 1, y);
}

//       
void FillGameBoard ()
{
	CursorX = -1;
	CursorY = -1;
	SelectX = -1;
	SelectY = -1;
	Summ = 0;
	//     
	srand (GetTickCount ());
	ZeroMemory (GameBoard, sizeof (GameBoard));
	FillGameBoardCell (0, 0);
	//  
	DrawGameBoard ();
	//    
	GetLocalTime (&stStart);
	GameComplete = false;
}

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_CREATE:
		{
			//    
			hbrNormal = CreateSolidBrush (RGB (255, 255, 255));
			hbrSelected = CreateHatchBrush (HS_DIAGCROSS, RGB (128, 128, 255));
			hbrBackground = CreateSolidBrush (RGB (128, 128, 255));
			hbrCursor = CreateSolidBrush (RGB (0, 0, 255));
			//    
			LOGFONT lf;
			ZeroMemory (&lf, sizeof (lf));
			lf.lfHeight = - CELL_SIZE;
			lf.lfQuality = ANTIALIASED_QUALITY;
			lstrcpy (lf.lfFaceName, "Arial");
			hfntCell = CreateFontIndirect (&lf);
			//     
			HDC hDC = GetDC (hWnd);
			hMemDC = CreateCompatibleDC (hDC);
			hMemBitmap = CreateCompatibleBitmap (hDC,
				BOARD_HEIGHT * CELL_SIZE, BOARD_WIDTH * CELL_SIZE);
			hMemBitmapOld = SelectBitmap (hMemDC, hMemBitmap);
            ReleaseDC (hWnd, hDC);
			VictoryCount = 0;
			//    
			FillGameBoard ();
			//   
			SetTimer (hWnd, 100, 1000, NULL);
			return 0;
		}

		case WM_TIMER:
			//     
			if (GameComplete) {
				SetWindowText (hWnd, szTitle);
			} else {
				int elapsed = GetGameTime ();
				char txt [256];
				wsprintf (txt, "%s - %02d:%02d", szTitle, elapsed / 60, elapsed % 60);
				SetWindowText (hWnd, txt);
			}
			break;

		case WM_PAINT:
			//  
			if (GetUpdateRect (hWnd, NULL, FALSE)) {
				PAINTSTRUCT ps;
				BeginPaint (hWnd, &ps);
				//   
				BitBlt (ps.hdc, 0, 0, BOARD_WIDTH * CELL_SIZE, BOARD_HEIGHT * CELL_SIZE,
					hMemDC, 0, 0, SRCCOPY);
				EndPaint (hWnd, &ps);
			}
			return 0;

		case WM_LBUTTONDOWN:
		{
			//     
			int x = GET_X_LPARAM (lParam) / CELL_SIZE; 
			int y = GET_Y_LPARAM (lParam) / CELL_SIZE; 
			GameComplete = ClickCell (x, y);
			InvalidateRect (hWnd, NULL, FALSE);
			UpdateWindow (hWnd);
			if (GameComplete) {
				//  
				int elapsed = GetGameTime ();
				char txt [256];
				wsprintf (txt, "!   %02d:%02d. "
					" ! :-) ...  .\n"
					"      ...",
					elapsed / 60, elapsed % 60);
				MessageBox (hWnd, txt, szTitle, MB_OK | MB_ICONEXCLAMATION);
				++VictoryCount;
				FillGameBoard ();
				InvalidateRect (hWnd, NULL, TRUE);
				UpdateWindow (hWnd);
			}
			break;
		}

		case WM_COMMAND:
			switch (LOWORD (wParam)) {
				case ID_RULES:
					//    " "
					MessageBox (hWnd, " :\n"
						"     ,     .\n"
						"      , "
						"    10.\n"
						"!    ,     . :-)\n"
						"    ,     ...",
						szTitle, MB_OK | MB_ICONINFORMATION);
					return 0;
				case ID_NEWGAME:
					//    "  "
					FillGameBoard ();
					InvalidateRect (hWnd, NULL, TRUE);
					UpdateWindow (hWnd);
					return 0;
				case ID_EXIT:
					//    "  "
					DestroyWindow (hWnd);
					return 0;
			}
			break;

		case WM_DESTROY:
			//  ,  
			KillTimer (hWnd, 100);
			DeleteBrush (hbrNormal);
			DeleteBrush (hbrSelected);
			DeleteBrush (hbrBackground);
			DeleteBrush (hbrCursor);
			DeleteFont (hfntCell);
			SelectBitmap (hMemDC, hMemBitmapOld);
			DeleteDC (hMemDC);
			DeleteBitmap (hMemBitmap);
			PostQuitMessage (0);
			return 0;
	}
	return DefWindowProc (hWnd, message, wParam, lParam);
}

int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE, LPTSTR, int)
{
	for (int i = 0; i < BACK_SIZE; ++i)
		hbmpImg [i] = (HBITMAP) LoadImage (hInstance, MAKEINTRESOURCE (IDB_IMG1 + i),
			IMAGE_BITMAP, 300, 300, LR_CREATEDIBSECTION);
	//   
	WNDCLASSEX wcex;
	ZeroMemory (&wcex, sizeof (wcex));
	wcex.cbSize = sizeof (WNDCLASSEX); 
	wcex.lpfnWndProc = WndProc;
	wcex.hInstance = hInstance;
	wcex.hIcon = LoadIcon (NULL, IDI_ASTERISK);
	wcex.hCursor = LoadCursor (NULL, IDC_ARROW);
	wcex.hbrBackground = NULL;
	wcex.lpszClassName = "lb041227";
	if (RegisterClassEx (&wcex)) {
		//    
		const DWORD dwStyle = WS_OVERLAPPED | WS_BORDER | WS_CAPTION | WS_SYSMENU |
			WS_MINIMIZEBOX | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE;
		const DWORD dwExStyle = WS_EX_APPWINDOW;
		RECT rc = {0, 0, BOARD_HEIGHT * CELL_SIZE - 1, BOARD_WIDTH * CELL_SIZE - 1};
		AdjustWindowRectEx (&rc, dwStyle, TRUE, dwExStyle);
		//  
		HMENU hMenu = CreateMenu ();		
		AppendMenu (hMenu, 0, ID_RULES, "& ");
		AppendMenu (hMenu, 0, ID_NEWGAME, "&  ");
		AppendMenu (hMenu, 0, ID_EXIT, "&  ");
		//  
		HWND hWnd = CreateWindowEx (dwExStyle, wcex.lpszClassName, szTitle,
			dwStyle, CW_USEDEFAULT, CW_USEDEFAULT, rc.right - rc.left, rc.bottom - rc.top,
			GetDesktopWindow (), hMenu, hInstance, NULL);
		if (hWnd) {
			//  
			ShowWindow (hWnd, SW_NORMAL);
			UpdateWindow (hWnd);
			//   
			MSG msg;
			while (GetMessage (&msg, NULL, 0, 0)) {
				TranslateMessage (&msg);
				DispatchMessage (&msg);
			}
		}
	}
	for (i = 0; i < BACK_SIZE; ++i)
		DeleteBitmap (hbmpImg [i]);
	return 0;
}
